@@ -18,12 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
+#include <libglom/libglom_config.h>
+ 
 #include <libglom/connectionpool.h>
 #include <libglom/document/document.h>
 #include <libglom/utils.h>
 //#include <libgdamm/connectionevent.h>
-#include <libglom/libglom_config.h>
 #include <glibmm/i18n.h>
 
 #ifdef G_OS_WIN32
@@ -644,7 +644,7 @@ bool ConnectionPool::startup(const SlotProgress& slot_progress, bool network_sha
 
   //If we crash while running (unlikely, hopefully), then try to cleanup.
   //Comment this out if you want to see the backtrace in a debugger.
-  previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
+  //previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
 
   return true;
 }
